"use strict";

import { Canvas, LinearGradient, RoundedRect, Shadow } from '@shopify/react-native-skia';
import { createOuterShadowOffset, createStyles, getLinearDirection } from "../utils.js";
import { useMemo } from 'react';

/**
 * LinearShadowCanvas
 * -------------------
 * A Skia-based component that renders a RoundedRect with a
 * linear gradient fill. It also applies an inner shadow
 * and (optionally) a reflected light effect.
 */
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
export default function LinearShadowCanvas({
  width,
  height,
  style,
  shadowColor,
  shadowOffset,
  shadowBlur,
  inset,
  reflectedLightColor,
  reflectedLightOffset,
  reflectedLightBlur,
  from = 'top',
  to = 'bottom',
  colors = ['#FFFFFF', '#FFFFFF'],
  ...props
}) {
  const isReflectedLightEnabled = props.isReflectedLightEnabled !== undefined ? props.isReflectedLightEnabled : inset;
  const boxRadius = Number(style ? style.borderRadius : 0) || 0;
  const {
    start,
    end
  } = getLinearDirection({
    width,
    height,
    from,
    to
  });
  const styles = createStyles({
    width,
    height
  });
  const outerShadowOffset = useMemo(() => createOuterShadowOffset({
    inset,
    shadowColor,
    shadowBlur,
    shadowOffset
  }), [inset, shadowColor, shadowBlur, shadowOffset]);
  return /*#__PURE__*/_jsx(Canvas, {
    style: [outerShadowOffset, style, styles.canvas],
    children: /*#__PURE__*/_jsxs(RoundedRect, {
      x: 0,
      y: 0,
      width: width,
      height: height,
      r: boxRadius,
      children: [/*#__PURE__*/_jsx(LinearGradient, {
        start: start,
        end: end,
        colors: colors
      }), inset && /*#__PURE__*/_jsx(Shadow, {
        dx: shadowOffset.width,
        dy: shadowOffset.height,
        blur: shadowBlur,
        color: shadowColor,
        inner: true
      }), isReflectedLightEnabled && /*#__PURE__*/_jsx(Shadow, {
        dx: reflectedLightOffset.width,
        dy: reflectedLightOffset.height,
        blur: reflectedLightBlur,
        color: reflectedLightColor,
        inner: true
      })]
    })
  });
}
//# sourceMappingURL=LinearShadowCanvas.js.map